`timescale 1ns / 1ps

/**
This module implements a 24-bit CRC of 8-bit data.  The
generator polynomial is
24+23+18+17+14+11+10+7+6+5+4+3+1+0
which is from open-PGP.

Implementation from kishalay@metreoncreations.com.\

changed to synchronous reset (Drew Baden, 9/1/04)
*/

module crc24(CLK,RESET,LOAD,DATAIN,CRCOUT);
	
	input CLK, RESET, LOAD;
	input [7:0] DATAIN;
	output [23:0] CRCOUT;
	reg [23:0] CRCOUT;

always @(posedge CLK or posedge RESET) begin
	if (RESET) CRCOUT<=24'hffffff; 
	else if (LOAD) begin
		CRCOUT[0]<=DATAIN[0] ^ CRCOUT[21] ^ CRCOUT[20] ^ CRCOUT[19] ^ CRCOUT[18] ^ CRCOUT[17] ^ CRCOUT[16];
		CRCOUT[1]<=DATAIN[1] ^ CRCOUT[16] ^ CRCOUT[22];
		CRCOUT[2]<=DATAIN[2] ^ CRCOUT[17] ^ CRCOUT[23];
		CRCOUT[3]<=DATAIN[3] ^ CRCOUT[21] ^ CRCOUT[20] ^ CRCOUT[19] ^ CRCOUT[17] ^ CRCOUT[16];
		CRCOUT[4]<=DATAIN[4] ^ CRCOUT[22] ^ CRCOUT[19] ^ CRCOUT[16];
		CRCOUT[5]<=DATAIN[5] ^ CRCOUT[23] ^ CRCOUT[21] ^ CRCOUT[19] ^ CRCOUT[18] ^ CRCOUT[16];
		CRCOUT[6]<=DATAIN[6] ^ CRCOUT[22] ^ CRCOUT[21] ^ CRCOUT[18] ^ CRCOUT[16];
		CRCOUT[7]<=DATAIN[7] ^ CRCOUT[23] ^ CRCOUT[22] ^ CRCOUT[21] ^ CRCOUT[20] ^ CRCOUT[18] ^ CRCOUT[16];
		CRCOUT[8]<=CRCOUT[23] ^ CRCOUT[22] ^ CRCOUT[21] ^ CRCOUT[19] ^ CRCOUT[17] ^ CRCOUT[0];
		CRCOUT[9]<=CRCOUT[23] ^ CRCOUT[22] ^ CRCOUT[20] ^ CRCOUT[18] ^ CRCOUT[1];
		CRCOUT[10]<=CRCOUT[23] ^ CRCOUT[20] ^ CRCOUT[18] ^ CRCOUT[17] ^ CRCOUT[16] ^ CRCOUT[2];
		CRCOUT[11]<=CRCOUT[20] ^ CRCOUT[16] ^ CRCOUT[3];
		CRCOUT[12]<=CRCOUT[21] ^ CRCOUT[17] ^ CRCOUT[4];
		CRCOUT[13]<=CRCOUT[22] ^ CRCOUT[18] ^ CRCOUT[5];
		CRCOUT[14]<=CRCOUT[23] ^ CRCOUT[21] ^ CRCOUT[20] ^ CRCOUT[18] ^ CRCOUT[17] ^ CRCOUT[16] ^ CRCOUT[6];
		CRCOUT[15]<=CRCOUT[22] ^ CRCOUT[21] ^ CRCOUT[19] ^ CRCOUT[18] ^ CRCOUT[17] ^ CRCOUT[7];
		CRCOUT[16]<=CRCOUT[23] ^ CRCOUT[22] ^ CRCOUT[20] ^ CRCOUT[19] ^ CRCOUT[18] ^ CRCOUT[8];
		CRCOUT[17]<=CRCOUT[23] ^ CRCOUT[18] ^ CRCOUT[17] ^ CRCOUT[16] ^ CRCOUT[9];
		CRCOUT[18]<=CRCOUT[21] ^ CRCOUT[20] ^ CRCOUT[16] ^ CRCOUT[10];
		CRCOUT[19]<=CRCOUT[22] ^ CRCOUT[21] ^ CRCOUT[17] ^ CRCOUT[11];
		CRCOUT[20]<=CRCOUT[23] ^ CRCOUT[22] ^ CRCOUT[18] ^ CRCOUT[12];
		CRCOUT[21]<=CRCOUT[23] ^ CRCOUT[19] ^ CRCOUT[13];
		CRCOUT[22]<=CRCOUT[20] ^ CRCOUT[14];
		CRCOUT[23]<=CRCOUT[20] ^ CRCOUT[19] ^ CRCOUT[18] ^ CRCOUT[17] ^ CRCOUT[16] ^ CRCOUT[15];
	end
end

endmodule
