`timescale 1ns / 1ps
/*Module copied and adapted from HTR Altera project.
 Changes from HTR Altera:
 - changed syntax into Verilog2001 style.
 - signals for the second Xilinx and Flash device not hooked-up on top level.
// State machine to load each of the 2 xilinx sequentially.
// It is executed only once after power-up.
*/

module load2xi(    
   input clock, enable1, enable2, done1, done2,
   output reg do1, do2   	);

 reg[2:0] state;
 parameter[2:0] WAIT=0, DO1=1, DO1_1=2, COUNT=3,
			   DO2=4, DO2_1=5, WAIT_CLEAR=6;

 reg[16:0] counter;
 wire counter_done = (counter == 16'hFFFF);
always @ (posedge clock)
    case (state)
		WAIT: begin
			do1 <= #1 0;	do2 <= #1 0;
			counter <= #1 0;
         if (enable1) state <= #1 DO1;
         else if (enable2) state <= #1 DO2;
         else state <= #1 WAIT_CLEAR;
			end  // WAIT
		DO1: begin
			do1 <= #1 1;
			state <= #1 DO1_1;
			end
		DO1_1: begin
			if (done1) begin
				if (enable2) state <= #1 COUNT;
				else state <= #1 WAIT_CLEAR;
				end
			else state <= #1 DO1_1;
			end
		COUNT: begin
			do1 <= #1 0;
			counter <= #1 counter + 1;
			if (counter_done) state <= #1 DO2;
			else state <= #1 COUNT;
			end
		DO2: begin
			do2 <= #1 1;
			state <= #1 DO2_1;
			end
		DO2_1: begin
			if (done2) state <= #1 WAIT_CLEAR;
			else state <= #1 DO2_1;
			end
		WAIT_CLEAR: begin
			do2 <= #1 0;
			state <= #1 WAIT_CLEAR;
			end
	endcase

endmodule
